@@ -34,6 +34,13 @@ class AttrData:
         self.polling_period = -1
         self.memorized = False
         self.hw_memorized = False
+        self.alarm_event_implemented = False
+        self.alarm_event_detect = False
+        self.archive_event_implemented = False
+        self.archive_event_detect = False
+        self.change_event_implemented = False
+        self.change_event_detect = False
+        self.data_ready_event_implemented = False
         if name is None:
             self.read_method_name = None
             self.write_method_name = None
@@ -113,6 +120,18 @@ class AttrData:
             self.attr_class = FwdAttr
             self.attr_args = [self.attr_name]
 
+        self.alarm_event_implemented = attr_dict.pop("alarm_event_implemented", False)
+        self.alarm_event_detect = attr_dict.pop("alarm_event_detect", False)
+        self.archive_event_implemented = attr_dict.pop(
+            "archive_event_implemented", False
+        )
+        self.archive_event_detect = attr_dict.pop("archive_event_detect", False)
+        self.change_event_implemented = attr_dict.pop("change_event_implemented", False)
+        self.change_event_detect = attr_dict.pop("change_event_detect", False)
+        self.data_ready_event_implemented = attr_dict.pop(
+            "data_ready_event_implemented", False
+        )
+
         if self.forward:
             self.att_prop = self.__create_user_default_fwdattr_prop(attr_dict)
         elif len(attr_dict):
@@ -336,6 +355,20 @@ class AttrData:
         else:
             self.memorized = False
 
+        self.alarm_event_implemented = extra_info.get("alarm_event_implemented", False)
+        self.alarm_event_detect = extra_info.get("alarm_event_detect", False)
+        self.archive_event_implemented = extra_info.get(
+            "archive_event_implemented", False
+        )
+        self.archive_event_detect = extra_info.get("archive_event_detect", False)
+        self.change_event_implemented = extra_info.get(
+            "change_event_implemented", False
+        )
+        self.change_event_detect = extra_info.get("change_event_detect", False)
+        self.data_ready_event_implemented = extra_info.get(
+            "data_ready_event_implemented", False
+        )
+
         if self.attr_type == CmdArgType.DevEnum:
             if "enum_labels" not in extra_info:
                 throw_ex(
