@@ -9,6 +9,7 @@ from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
 
 # Tango imports
 from tango.green import AbstractExecutor, get_ident
+from tango.utils import _get_current_otel_context, _get_non_tango_source_location
 
 __all__ = (
     "FuturesExecutor",
@@ -73,6 +74,9 @@ class FuturesExecutor(AbstractExecutor):
 
     def delegate(self, fn, *args, **kwargs):
         """Return the given operation as a concurrent future."""
+        if hasattr(fn, "__trace_kwargs__"):
+            kwargs["trace_location"] = _get_non_tango_source_location()
+            kwargs["trace_context"] = _get_current_otel_context()
         return self.subexecutor.submit(fn, *args, **kwargs)
 
     def access(self, accessor, timeout=None):
