@@ -9,15 +9,13 @@ import json
 import shutil
 from functools import partial
 from subprocess import Popen
-from time import sleep
 
 import pytest
 
-from tango import DevFailed, DeviceProxy, GreenMode, Util
+from tango import DeviceProxy, GreenMode, Util
 from tango.asyncio import DeviceProxy as asyncio_DeviceProxy
 from tango.gevent import DeviceProxy as gevent_DeviceProxy
 from tango.futures import DeviceProxy as futures_DeviceProxy
-from tango.test_context import get_server_port_via_pid
 from tango.test_utils import (
     ClassicAPISimpleDeviceClass,
     ClassicAPISimpleDeviceImpl,
@@ -34,6 +32,7 @@ from tango.test_utils import (
     server_green_mode,
     server_serial_model,
     state,
+    wait_for_nodb_proxy_via_pid,
 )
 
 from tango._tango import _dump_cpp_coverage
@@ -162,33 +161,9 @@ def start_server(host, server, inst, device):
     return proc
 
 
-def get_proxy(host, port, device, green_mode):
-    access = f"tango://{host}:{port}/{device}#dbase=no"
-    return device_proxy_map[green_mode](access)
-
-
-def wait_for_proxy(host, proc, device, green_mode, retries=400, delay=0.03):
-    port = get_server_port_via_pid(proc.pid, host, retries, delay)
-    if port is not None:
-        count = 0
-        while count < retries:
-            try:
-                proxy = get_proxy(host, port, device, green_mode)
-                proxy.ping()
-                proxy.state()
-                return proxy
-            except DevFailed as exc:
-                last_error = str(exc)
-                sleep(delay)
-            count += 1
-    raise RuntimeError(
-        f"Device {device} did not start up within {count * delay:.1f} sec!\n"
-        f"Last error: {last_error}."
-    )
-
-
 @pytest.fixture(
     params=GreenMode.values.values(),
+    ids=str,
     scope="module",
 )
 def tango_test_with_green_modes(request):
@@ -198,7 +173,9 @@ def tango_test_with_green_modes(request):
     device = "sys/tg_test/17"
     host = "127.0.0.1"
     proc = start_server(host, server, inst, device)
-    proxy = wait_for_proxy(host, proc, device, green_mode)
+    proxy = wait_for_nodb_proxy_via_pid(
+        proc.pid, host, device, device_proxy_map[green_mode]
+    )
 
     yield proxy
 
@@ -214,7 +191,9 @@ def tango_test():
     device = "sys/tg_test/17"
     host = "127.0.0.1"
     proc = start_server(host, server, inst, device)
-    proxy = wait_for_proxy(host, proc, device, green_mode)
+    proxy = wait_for_nodb_proxy_via_pid(
+        proc.pid, host, device, device_proxy_map[green_mode]
+    )
 
     yield proxy
 
@@ -229,7 +208,9 @@ def tango_test_process_device_trl_with_function_scope():
     device = "sys/tg_test/18"
     host = "127.0.0.1"
     proc = start_server(host, server, inst, device)
-    proxy = wait_for_proxy(host, proc, device, green_mode)
+    proxy = wait_for_nodb_proxy_via_pid(
+        proc.pid, host, device, device_proxy_map[green_mode]
+    )
 
     device_trl = (
         f"tango://{proxy.get_dev_host()}:{proxy.get_dev_port()}/"
@@ -242,6 +223,7 @@ def tango_test_process_device_trl_with_function_scope():
 
 @pytest.fixture(
     params=GreenMode.values.values(),
+    ids=str,
     scope="module",
 )
 def green_mode_device_proxy(request):
@@ -292,11 +274,11 @@ def mixed_tango_test_server():
     process.start()
 
     proxy_waiter = partial(
-        wait_for_proxy,
+        wait_for_nodb_proxy_via_pid,
+        process.pid,
         "127.0.0.1",
-        process,
         "dserver/mixedserver/1",
-        GreenMode.Synchronous,
+        device_proxy_map[GreenMode.Synchronous],
     )
     yield process, proxy_waiter
 
