@@ -1,5 +1,6 @@
 # SPDX-FileCopyrightText: All Contributors to the PyTango project
 # SPDX-License-Identifier: LGPL-3.0-or-later
+import time
 import numpy as np
 
 try:
@@ -21,6 +22,7 @@ from tango import (
 )
 from tango.server import Device
 from tango.server import command
+from tango.pyutil import TimedCmdData
 from tango.test_utils import DeviceTestContext
 from tango.test_utils import (
     assert_close,
@@ -30,7 +32,7 @@ from tango.test_utils import (
 )
 
 
-def test_identity_command(command_typed_values, server_green_mode):
+def test_static_commands(command_typed_values, server_green_mode):
     dtype, values, expected = command_typed_values
 
     if dtype == (bool,):
@@ -40,6 +42,19 @@ def test_identity_command(command_typed_values, server_green_mode):
 
         class TestDevice(Device):
             green_mode = server_green_mode
+            _last_value = None
+
+            @command()
+            async def void_cmd(self):
+                pass
+
+            @command(dtype_in=dtype)
+            async def no_return_cmd(self, arg):
+                self._last_value = arg
+
+            @command(dtype_out=dtype)
+            async def no_input_cmd(self):
+                return self._last_value
 
             @command(dtype_in=dtype, dtype_out=dtype)
             async def identity(self, arg):
@@ -49,6 +64,19 @@ def test_identity_command(command_typed_values, server_green_mode):
 
         class TestDevice(Device):
             green_mode = server_green_mode
+            _last_value = None
+
+            @command()
+            def void_cmd(self):
+                pass
+
+            @command(dtype_in=dtype)
+            def no_return_cmd(self, arg):
+                self._last_value = arg
+
+            @command(dtype_out=dtype)
+            def no_input_cmd(self):
+                return self._last_value
 
             @command(dtype_in=dtype, dtype_out=dtype)
             def identity(self, arg):
@@ -56,10 +84,14 @@ def test_identity_command(command_typed_values, server_green_mode):
 
     with DeviceTestContext(TestDevice) as proxy:
         for value in values:
+            # just check, that we can execute it
+            proxy.void_cmd()
+            proxy.no_return_cmd(value)
+            assert_close(proxy.no_input_cmd(), expected(value))
             assert_close(proxy.identity(value), expected(value))
 
 
-def test_identity_command_with_typing(command_typed_values):
+def test_static_commands_with_typing(command_typed_values):
     dtype, values, expected = command_typed_values
     tuple_hint, list_hint, _, _ = convert_dtype_to_typing_hint(dtype)
 
@@ -67,10 +99,22 @@ def test_identity_command_with_typing(command_typed_values):
         pytest.xfail("Not supported for some reasons")
 
     class TestDevice(Device):
+        _last_arg = None
+
         @command()
         def command_tuple_hint(self, arg: tuple_hint) -> tuple_hint:
             return arg
 
+        @general_decorator
+        @command()
+        def command_with_decorator_before(self, arg: tuple_hint) -> tuple_hint:
+            return arg
+
+        @command()
+        @general_decorator
+        def command_with_decorator_after(self, arg: tuple_hint) -> tuple_hint:
+            return arg
+
         @command()
         def command_list_hint(self, arg: list_hint) -> list_hint:
             return arg
@@ -79,11 +123,46 @@ def test_identity_command_with_typing(command_typed_values):
         def command_user_type_has_priority(self, arg: dict) -> dict:
             return arg
 
+        @command(dtype_in=dtype)
+        def command_in_type_dtype_out_type_hint(self, arg) -> tuple_hint:
+            return arg
+
+        @command(dtype_out=dtype)
+        def command_in_type_hint_out_type_dtype(self, arg: tuple_hint):
+            return arg
+
+        @command()
+        def command_only_in_type_hint(self, arg: tuple_hint):
+            self._last_arg = arg
+
+        @command()
+        def command_only_out_type_hint(self) -> tuple_hint:
+            return self._last_arg
+
+        @command()
+        def command_arg_list_in_only_out_type_hint(self, *args) -> tuple_hint:
+            return self._last_arg
+
+        @command()
+        def reset(self):
+            self._last_arg = None
+
     with DeviceTestContext(TestDevice) as proxy:
         for value in values:
             assert_close(proxy.command_tuple_hint(value), expected(value))
+            assert_close(proxy.command_with_decorator_before(value), expected(value))
+            assert_close(proxy.command_with_decorator_after(value), expected(value))
             assert_close(proxy.command_list_hint(value), expected(value))
             assert_close(proxy.command_user_type_has_priority(value), expected(value))
+            assert_close(
+                proxy.command_in_type_dtype_out_type_hint(value), expected(value)
+            )
+            proxy.command_only_in_type_hint(value)
+            assert_close(proxy.command_only_out_type_hint(), expected(value))
+            assert_close(
+                proxy.command_arg_list_in_only_out_type_hint(), expected(value)
+            )
+            proxy.reset()
 
 
 def test_devstate_command_with_typing():
@@ -338,12 +417,25 @@ def test_dynamic_command(device_command_level, server_green_mode):
 
     class BaseTestDevice(Device):
         green_mode = server_green_mode
+        _last_in = None
 
         def __init__(self, *args, **kwargs):
             super().__init__(*args, **kwargs)
             self._is_allowed = True
 
         def _add_dyn_cmd(self):
+            cmd = command(f=self.void_cmd)
+            self.add_command(cmd, device_command_level)
+
+            cmd = command(f=self.no_return_cmd, dtype_in=int)
+            self.add_command(cmd, device_command_level)
+
+            cmd = command(f=self.no_input_cmd, dtype_out=int)
+            self.add_command(cmd, device_command_level)
+
+            cmd = command(f=self.no_input_args_list_cmd, dtype_out=int)
+            self.add_command(cmd, device_command_level)
+
             cmd = command(f=self.identity, dtype_in=int, dtype_out=int)
             self.add_command(cmd, device_command_level)
 
@@ -393,6 +485,18 @@ def test_dynamic_command(device_command_level, server_green_mode):
     if server_green_mode == GreenMode.Asyncio:
 
         class TestDevice(BaseTestDevice):
+            async def void_cmd(self):
+                pass
+
+            async def no_return_cmd(self, arg):
+                self._last_in = arg
+
+            async def no_input_cmd(self):
+                return self._last_in
+
+            async def no_input_args_list_cmd(self, *args):
+                return self._last_in
+
             async def identity(self, arg):
                 return arg
 
@@ -425,6 +529,18 @@ def test_dynamic_command(device_command_level, server_green_mode):
     else:
 
         class TestDevice(BaseTestDevice):
+            def void_cmd(self):
+                pass
+
+            def no_return_cmd(self, arg):
+                self._last_in = arg
+
+            def no_input_cmd(self):
+                return self._last_in
+
+            def no_input_args_list_cmd(self, *args):
+                return self._last_in
+
             def identity(self, arg):
                 return arg
 
@@ -462,6 +578,12 @@ def test_dynamic_command(device_command_level, server_green_mode):
         async_is_allowed_callable_class.make_allowed(True)
         is_allowed = True
 
+        # check that cmd exist and can be executed
+        proxy.void_cmd()
+
+        proxy.no_return_cmd(1)
+        assert_close(proxy.no_input_cmd(), 1)
+
         assert_close(proxy.identity(1), 1)
         assert_close(proxy.identity_kwarg_string(1), 1)
         assert_close(proxy.identity_kwarg_callable(1), 1)
@@ -509,6 +631,9 @@ def test_identity_dynamic_command_with_typing(command_typed_values):
         def command_user_type_has_priority(self, arg: dict) -> dict:
             return arg
 
+        def command_args_list_tuple_hint(self, *args) -> tuple_hint:
+            return values[0]
+
         @command()
         def add_dyn_cmd(self):
             cmd = command(f=self.command_tuple_hint)
@@ -522,12 +647,16 @@ def test_identity_dynamic_command_with_typing(command_typed_values):
             )
             self.add_command(cmd)
 
+            cmd = command(f=self.command_args_list_tuple_hint)
+            self.add_command(cmd)
+
     with DeviceTestContext(TestDevice) as proxy:
         proxy.add_dyn_cmd()
         for value in values:
             assert_close(proxy.command_tuple_hint(value), expected(value))
             assert_close(proxy.command_list_hint(value), expected(value))
             assert_close(proxy.command_user_type_has_priority(value), expected(value))
+        assert_close(proxy.command_args_list_tuple_hint(), expected(values[0]))
 
 
 if npt:
@@ -651,3 +780,200 @@ def test_command_info():
         assert info[4].in_type_desc == "identity_spectrum doc_in"
         assert info[4].out_type == DevVarDoubleArray
         assert info[4].out_type_desc == "identity_spectrum doc_out"
+
+
+def test_docstring_parsing():
+
+    pytest.importorskip("docstring_parser")
+
+    class TestDevice(Device):
+
+        @command
+        def rest_docstring(self, val_in: int) -> int:
+            """
+            :param val_in: The input integer value.
+            :type val_in: int
+            :return: An integer result after processing.
+            :rtype: int
+            """
+            return val_in
+
+        @command
+        def rest_docstring_only_in(self, val_in: int):
+            """
+            :param val_in: The input integer value.
+            :type val_in: int
+            """
+            pass
+
+        @command
+        def rest_docstring_only_out(self) -> int:
+            """
+            :return: An integer result after processing.
+            :rtype: int
+            """
+            pass
+
+        @command
+        def rest_docstring_args_list_only_out(self, *args) -> int:
+            """
+            :return: An integer result after processing.
+            :rtype: int
+            """
+            pass
+
+        @command
+        def google_docstring(self, val_in: int) -> int:
+            """
+            Args:
+                val_in (int): The input integer value.
+
+            Returns:
+                int: An integer result after processing.
+            """
+            return val_in
+
+        @command
+        def numpy_docstring(self, val_in: int) -> int:
+            """
+            Parameters
+            ----------
+            val_in : int
+                The input integer value.
+
+            Returns
+            -------
+            int
+                An integer result after processing.
+            """
+            return val_in
+
+        @command
+        def epydoc_docstring(self, val_in: int) -> int:
+            """
+            @param val_in: The input integer value.
+            @type val_in: int
+            @return: An integer result after processing.
+            @rtype: int
+            """
+            return val_in
+
+        @command(
+            doc_in="val_in (int): The input integer value.",
+            doc_out="returns (int): An integer result after processing.",
+        )
+        def kwargs_have_priority_over_docstring(self, val_in: int) -> int:
+            """
+            :param val_in: THIS DESCRIPTION WON'T BE USED
+            :type val_in: int
+            :return: THIS DESCRIPTION WON'T BE USED
+            :rtype: int
+            """
+            return val_in
+
+        @command
+        def no_docstring(self, val_in: int) -> int:
+            return val_in
+
+        @command
+        def no_docstring_args_list_in(self, *args) -> int:
+            return 123
+
+        @command
+        def no_docstring_only_in(self, val_in: int): ...
+
+        @command
+        def no_docstring_only_out(self) -> int:
+            return 123
+
+        @command
+        def no_docstring_no_in_no_out(self): ...
+
+    with DeviceTestContext(TestDevice) as proxy:
+
+        expected_int_in_desc = "val_in (int): The input integer value."
+        expected_int_out_desc = "returns (int): An integer result after processing."
+        expected_void_in_desc = "No input parameter (DevVoid)"
+        expected_void_out_desc = "No output parameter (DevVoid)"
+
+        for cmd_name in [
+            "rest_docstring",
+            "google_docstring",
+            "numpy_docstring",
+            "epydoc_docstring",
+            "kwargs_have_priority_over_docstring",
+        ]:
+            cfg = proxy.get_command_config(cmd_name)
+            assert cfg.in_type_desc == expected_int_in_desc
+            assert cfg.out_type_desc == expected_int_out_desc
+
+        cfg = proxy.get_command_config("rest_docstring_only_in")
+        assert cfg.in_type_desc == expected_int_in_desc
+        assert cfg.out_type_desc == expected_void_out_desc
+
+        cfg = proxy.get_command_config("rest_docstring_only_out")
+        assert cfg.in_type_desc == expected_void_in_desc
+        assert cfg.out_type_desc == expected_int_out_desc
+
+        expected_int_in_desc = (
+            ":param val_in: (not documented)\n" ":type val_in: DevLong64"
+        )
+        expected_int_out_desc = ":return: (not documented)\n:rtype: DevLong64"
+
+        cfg = proxy.get_command_config("no_docstring")
+        assert cfg.in_type_desc == expected_int_in_desc
+        assert cfg.out_type_desc == expected_int_out_desc
+
+        cfg = proxy.get_command_config("no_docstring_args_list_in")
+        assert cfg.in_type_desc == expected_void_in_desc
+        assert cfg.out_type_desc == expected_int_out_desc
+
+        cfg = proxy.get_command_config("no_docstring_only_in")
+        assert cfg.in_type_desc == expected_int_in_desc
+        assert cfg.out_type_desc == expected_void_out_desc
+
+        cfg = proxy.get_command_config("no_docstring_only_out")
+        assert cfg.in_type_desc == expected_void_in_desc
+        assert cfg.out_type_desc == expected_int_out_desc
+
+        cfg = proxy.get_command_config("no_docstring_no_in_no_out")
+        assert cfg.in_type_desc == expected_void_in_desc
+        assert cfg.out_type_desc == expected_void_out_desc
+
+
+def test_fill_cmd_polling_buffer(command_typed_values):
+    dtype, values, expected = command_typed_values
+
+    start_time = time.time()
+
+    class TestDevice(Device):
+
+        @command(dtype_out=dtype)
+        def cmd(self):
+            return values[0]
+
+        @command
+        def fill_history(self):
+            data = []
+            for i, val in enumerate(values):
+                data.append(TimedCmdData(val, time_stamp=start_time + i))
+
+            data.append(
+                TimedCmdData(
+                    error=RuntimeError("Test"),
+                    time_stamp=start_time + len(data),
+                )
+            )
+            self.fill_cmd_polling_buffer("cmd", data)
+
+    with DeviceTestContext(TestDevice) as proxy:
+        # we do not want, that tango core fills up history automatically
+        proxy.poll_command("cmd", 0)
+        proxy.fill_history()
+        history = proxy.command_history("cmd", 10)
+        assert len(history) == len(values) + 1
+        for ind, cmd in enumerate(history[:-1]):
+            assert_close(cmd.extract(), expected(values[ind]))
+            assert_close(start_time + ind, cmd.get_date().totime())
+
+        assert history[-1].has_failed()
